'''

Description
-----------
    gg_knifetop addon adds knife ranking to the game.
    (http://forums.eventscripts.com/viewtopic.php?t=30980)

Commands and Cvars
------------------
    gg_knifetop                     - Enable plugin.
    
    say !ktop                       - Show ingame statisctics panel with top 10 knifers.

Requirements
------------
    Counter-Strike: Source
    EventScripts 2.0    
    GunGame 5 for EventScripts

Changelog
---------

Credits
-------
    Thanks to all GunGame 5 Team.
    (http://forums.eventscripts.com/viewtopic.php?t=27345).
    gg_knifetop is based on its source code.

'''

# ==============================================================================
#   IMPORTS
# ==============================================================================
# Python imports


# EventScripts imports
import es
import popuplib
import playerlib
import gamethread

# GunGame imports
import gungamelib
import ggknifetoplib

# ==============================================================================
#   ADDON REGISTRATION
# ==============================================================================
# Register this addon with EventScripts
info = es.AddonInfo()
info.name     = 'KnifeTop ranking system'
info.version  = '1.0'
info.url      = 'http://otstrel.ru'
info.basename = 'gungame/custom_addons/gg_knifetop'
info.author   = 'Otstrel.ru Team'

orderedKnifers = []

# ==============================================================================
#   GAME EVENTS
# ==============================================================================
def load():
    addon = gungamelib.registerAddon('gg_knifetop')
    addon.setDisplayName('GG Knife Top')
    addon.loadTranslationFile()
    
    # Load the knifers database into memory from file
    ggknifetoplib.loadKniferDatabase()
    
    # Loop through all the players
    for userid in es.getUseridList():
        # Get steamid
        steamid = gungamelib.getPlayer(userid).steamid
        
        # Is a bot?
        if 'BOT' in steamid:
            continue
        
        # Update timestamp
        if ggknifetoplib.getKnifeKills(steamid) or ggknifetoplib.getKnifeDeaths(steamid):
            ggknifetoplib.updateTimeStamp(steamid)
    
    # Build menus
    buildKnifeTopMenu()
    
    # Register commands
    addon.registerPublicCommand('ktop', displayKnifeTopMenu)
    
def unload():
    gungamelib.unregisterAddon('gg_knifetop')
    # Save the database
    ggknifetoplib.saveKniferDatabase()
    
# ==============================================================================
#   LOGIC
# ==============================================================================
def player_death(event_var):
    # ============
    # BASIC CHECKS
    # ============
    if event_var['weapon'] != 'knife':
        return
    
    if gungamelib.getGlobal('isWarmup'):
        return
    
    # ===============
    # GET PLAYER INFO
    # ===============
    attacker = int(event_var['attacker'])
    userid = int(event_var['userid'])
    userteam = event_var['es_userteam']
    attackerteam = event_var['es_attackerteam']
    
    # =============
    # SUICIDE CHECK
    # =============
    if (attackerteam == userteam) or (userid == attacker) or (attacker == 0):
        return

    # =================
    # GET ATTACKER INFO
    # =================
    gungameAttacker = gungamelib.getPlayer(attacker)
    
    # ===============
    # GET VICTIM INFO
    # ===============
    gungameVictim = gungamelib.getPlayer(userid)
    
    # =============
    # VICTIM CHECKS
    # =============
    # Is the victim AFK?
    if gungameVictim.isPlayerAFK():
        # gungamelib.msg('gg_knifetop', attacker, 'VictimAFK')
        return
    
    # Announce the level stealing
    # gungamelib.saytext2('gg_knifetop', '#all', gungameAttacker.index, 'KnifeKill', {'attacker': event_var['es_attackername'], 'victim': event_var['es_username']})

    addKnifeKill(gungameAttacker.steamid)
    addKnifeDeath(gungameVictim.steamid)
    
    # Prune old knifers and save the database
    ggknifetoplib.pruneKniferDatabase(gungamelib.getVariableValue('gg_prune_database'))
    ggknifetoplib.saveKniferDatabase()

    buildKnifeTopMenu()

def player_activate(event_var):
    # Get steamid
    steamid = gungamelib.getPlayer(event_var['userid']).steamid
        
    # Is a bot?
    if 'BOT' in steamid:
        return
    
    # Update their timestamp
    if ggknifetoplib.getKnifeKills(steamid) or ggknifetoplib.getKnifeDeaths(steamid):
        ggknifetoplib.updateTimeStamp(steamid)

def player_disconnect(event_var):
    userid = int(event_var['userid'])
    
    # Do not continue if the player does not exist
    if not gungamelib.playerExists(userid):
        return
    
    # Get steamid
    steamid = gungamelib.getPlayer(event_var['userid']).steamid
    
    # Is a bot?
    if 'BOT' in steamid:
        return
    
    # Update their timestamp
    if ggknifetoplib.getKnifeKills(steamid) or ggknifetoplib.getKnifeDeaths(steamid):
        ggknifetoplib.updateTimeStamp(steamid)

def displayKnifeTopMenu(userid):
    if gungamelib.getVariableValue('gg_stats'):
        gungamelib.sendOrderedMenu('knifetop', userid)

def buildKnifeTopMenu():
    global orderedKnifers
    orderedKnifers = ggknifetoplib.getOrderedKnifers()
    
    menu = gungamelib.OrderedMenu('knifetop', [], 10, prepKnifeTopMenu)
    menu.setTitle('GunGame: Knife Top Players')
    
    if not orderedKnifers:
        menu.addItem('No Knife Top Players Recorded')
    else:
        for knifer in orderedKnifers:
            # Get knifer information
            name = ggknifetoplib.getKniferName(knifer)
            kills = ggknifetoplib.getKnifeKills(knifer)
            deaths = ggknifetoplib.getKnifeDeaths(knifer)
        
            # Add menu item
            menu.addItem('%s: %s (%s/%s)' % (name, kills-deaths, kills, deaths))
    
    menu.buildMenu()

def prepKnifeTopMenu(userid, popupid):
    steamid = gungamelib.getPlayer(userid).steamid
    if steamid not in orderedKnifers:
        return
    
    rank = orderedKnifers.index(steamid) + 1
    page = int((rank - 1) / 10) + 1
    
    if popupid != 'OrderedMenu_knifetop:%s' % page:
        return
    
    lineNumber = rank - (page * 10) + 12 if page > 1 else rank + 2
    name = ggknifetoplib.getKniferName(steamid)
    kills = ggknifetoplib.getKnifeKills(steamid)
    deaths = ggknifetoplib.getKnifeDeaths(steamid)
            
    menu = popuplib.find(popupid)
    menu.modline(lineNumber, '->%i. %s: %s (%s/%s)' % (rank, name, kills-deaths, kills, deaths))
    gamethread.delayed(0, menu.modline, (lineNumber, '%i. %s: %s (%s/%s)' % (rank, name, kills-deaths, kills, deaths)))

def addKnifeKill(steamid):
    # Is a bot?
    if 'BOT' in steamid:
        return
    
    # Add knife kill to database
    ggknifetoplib.addKnifeKill(steamid)

def addKnifeDeath(steamid):
    # Is a bot?
    if 'BOT' in steamid:
        return
    
    # Add knife death to database
    ggknifetoplib.addKnifeDeath(steamid)
